#  Copyright (c) 2019 JD Williams
#
#  This file is part of Firefly, a Python SOA framework built by JD Williams. Firefly is free software; you can
#  redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 3 of the License, or (at your option) any later version.
#
#  Firefly is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
#  Public License for more details. You should have received a copy of the GNU Lesser General Public
#  License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  You should have received a copy of the GNU General Public License along with Firefly. If not, see
#  <http://www.gnu.org/licenses/>.

from __future__ import annotations

from abc import ABC, abstractmethod
from contextlib import contextmanager


class Mutex(ABC):
    @abstractmethod
    def acquire(self, key: str, timeout: int = None) -> bool:
        pass

    @abstractmethod
    def release(self, key: str):
        pass

    @contextmanager
    def __call__(self, key: str, timeout: int = None):
        self.acquire(key, timeout)
        try:
            yield
        finally:
            self.release(key)


class RateLimiter(ABC):
    @abstractmethod
    def acquire(self, key: str, max_concurrent: int, timeout: int = None) -> bool:
        pass

    @abstractmethod
    def release(self, key: str):
        pass

    @contextmanager
    def __call__(self, key: str, max_concurrent: int, timeout: int = None):
        self.acquire(key, max_concurrent, timeout=timeout)
        try:
            yield
        finally:
            self.release(key)
